############################ Copyrights and license ############################
#                                                                              #
# Copyright 2023 Christoph Reiter <reiter.christoph@gmail.com>                 #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Nicolas Schweitzer <nicolas.schweitzer@datadoghq.com>         #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2026 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

import unittest
from datetime import datetime, timedelta, timezone
from typing import Any
from unittest import mock

import github.Repository
import github.RepositoryDiscussion

from . import Framework

gho = Framework.github.GithubObject
ghusr = Framework.github.NamedUser
ghorg = Framework.github.Organization


class GithubObject(unittest.TestCase):
    def testAttributesAsRest(self):
        _ = gho.as_rest_api_attributes
        self.assertIsNone(_(None))
        self.assertDictEqual(_({}), {})
        self.assertDictEqual(_({"id": "NID", "databaseId": "DBID"}), {"node_id": "NID", "id": "DBID"})
        self.assertDictEqual(_({"someId": "someId"}), {"some_id": "someId"})
        self.assertDictEqual(_({"someObj": {"someId": "someId"}}), {"some_obj": {"some_id": "someId"}})
        self.assertDictEqual(_({"bodyHTML": "<html/>"}), {"body_html": "<html/>"})

    def testApiType(self):
        self.assertEqual(github.Repository.Repository.is_rest(), True)
        self.assertEqual(github.Repository.Repository.is_graphql(), False)

        self.assertEqual(github.RepositoryDiscussion.RepositoryDiscussion.is_rest(), False)
        self.assertEqual(github.RepositoryDiscussion.RepositoryDiscussion.is_graphql(), True)

    def testMakeUnionClassAttributeFromTypeName(self):
        req = mock.Mock(is_not_lazy=False)
        obj = TestingClass(req, {}, {})

        data = {"login": "login"}
        class_and_names = [(ghusr.NamedUser, "User"), (ghorg.Organization, "Organization")]

        def make(type_name: str | None, fallback_type: str | None = "User"):
            return obj._makeUnionClassAttributeFromTypeName(type_name, fallback_type, data, *class_and_names)

        none = make(None)
        usr = make("User")
        org = make("Organization")
        unknown = make("Unknown")
        bad = make("Unknown", None)

        self.assertIsInstance(none, gho._ValuedAttribute)
        self.assertIsInstance(usr, gho._ValuedAttribute)
        self.assertIsInstance(org, gho._ValuedAttribute)
        self.assertIsInstance(unknown, gho._ValuedAttribute)
        self.assertIsInstance(bad, gho._BadAttribute)

        self.assertIsNone(none.value)
        self.assertIsInstance(usr.value, ghusr.NamedUser)
        self.assertIsInstance(org.value, ghorg.Organization)
        self.assertIsInstance(unknown.value, ghusr.NamedUser)

        self.assertEqual(str(usr.value), 'NamedUser(login="login")')
        self.assertEqual(str(org.value), 'Organization(login="login")')
        self.assertEqual(str(unknown.value), 'NamedUser(login="login")')

    def testMakeUnionClassAttributeFromTypeKey(self):
        req = mock.Mock(is_not_lazy=False)
        obj = TestingClass(req, {}, {})

        class_and_names = [(ghusr.NamedUser, "User"), (ghorg.Organization, "Organization")]

        def make(data: dict[str, Any]):
            return obj._makeUnionClassAttributeFromTypeKey("type", "User", data, *class_and_names)

        default = make({"login": "login"})
        usr = make({"login": "login", "type": "User"})
        org = make({"login": "login", "type": "Organization"})
        unknown = make({"login": "login", "type": "Unknown"})

        self.assertIsInstance(default, gho._ValuedAttribute)
        self.assertIsInstance(usr, gho._ValuedAttribute)
        self.assertIsInstance(org, gho._ValuedAttribute)
        self.assertIsInstance(unknown, gho._ValuedAttribute)

        self.assertIsInstance(default.value, ghusr.NamedUser)
        self.assertIsInstance(usr.value, ghusr.NamedUser)
        self.assertIsInstance(org.value, ghorg.Organization)
        self.assertIsInstance(unknown.value, ghusr.NamedUser)

        self.assertEqual(str(default.value), 'NamedUser(login="login")')
        self.assertEqual(str(usr.value), 'NamedUser(login="login")')
        self.assertEqual(str(org.value), 'Organization(login="login")')
        self.assertEqual(str(unknown.value), 'NamedUser(login="login")')

    def testMakeUnionClassAttributeFromTypeKeyAndValueKey(self):
        req = mock.Mock(is_not_lazy=False)
        obj = TestingClass(req, {}, {})

        class_and_names = [(ghusr.NamedUser, "User"), (ghorg.Organization, "Organization")]

        def make(data: dict[str, Any]):
            return obj._makeUnionClassAttributeFromTypeKeyAndValueKey("type", "data", "User", data, *class_and_names)

        default = make({"data": {"login": "login"}})
        usr = make({"data": {"login": "login"}, "type": "User"})
        org = make({"data": {"login": "login"}, "type": "Organization"})
        unknown = make({"data": {"login": "login"}, "type": "Unknown"})

        self.assertIsInstance(default, gho._ValuedAttribute)
        self.assertIsInstance(usr, gho._ValuedAttribute)
        self.assertIsInstance(org, gho._ValuedAttribute)
        self.assertIsInstance(unknown, gho._ValuedAttribute)

        self.assertIsInstance(default.value, ghusr.NamedUser)
        self.assertIsInstance(usr.value, ghusr.NamedUser)
        self.assertIsInstance(org.value, ghorg.Organization)
        self.assertIsInstance(unknown.value, ghusr.NamedUser)

        self.assertEqual(str(default.value), 'NamedUser(login="login")')
        self.assertEqual(str(usr.value), 'NamedUser(login="login")')
        self.assertEqual(str(org.value), 'Organization(login="login")')
        self.assertEqual(str(unknown.value), 'NamedUser(login="login")')

    def testMakeDatetimeAttribute(self):
        for value, expected in [
            (None, None),
            (
                "2021-01-23T12:34:56Z",
                datetime(2021, 1, 23, 12, 34, 56, tzinfo=timezone.utc),
            ),
            (
                "2021-01-23T12:34:56+00:00",
                datetime(2021, 1, 23, 12, 34, 56, tzinfo=timezone.utc),
            ),
            (
                "2021-01-23T12:34:56+01:00",
                datetime(2021, 1, 23, 12, 34, 56, tzinfo=timezone(timedelta(hours=1))),
            ),
            (
                "2021-01-23T12:34:56-06:30",
                datetime(
                    2021,
                    1,
                    23,
                    12,
                    34,
                    56,
                    tzinfo=timezone(timedelta(hours=-6, minutes=-30)),
                ),
            ),
        ]:
            actual = gho.GithubObject._makeDatetimeAttribute(value)
            self.assertEqual(gho._ValuedAttribute, type(actual), value)
            self.assertEqual(expected, actual.value, value)

    def testMakeHttpDatetimeAttribute(self):
        for value, expected in [
            (None, None),
            # https://datatracker.ietf.org/doc/html/rfc7231#section-7.1.1.1
            (
                "Mon, 11 Sep 2023 14:07:29 GMT",
                datetime(2023, 9, 11, 14, 7, 29, tzinfo=timezone.utc),
            ),
            # obsolete formats:
            (
                "Monday, 11-Sep-23 14:07:29 GMT",
                datetime(2023, 9, 11, 14, 7, 29, tzinfo=timezone.utc),
            ),
            (
                "Mon Sep  11 14:07:29 2023",
                datetime(2023, 9, 11, 14, 7, 29, tzinfo=timezone.utc),
            ),
        ]:
            actual = gho.GithubObject._makeHttpDatetimeAttribute(value)
            self.assertEqual(gho._ValuedAttribute, type(actual), value)
            self.assertEqual(expected, actual.value, value)

    def testMakeHttpDatetimeAttributeBadValues(self):
        for value in ["not a timestamp", 1234]:
            actual = gho.GithubObject._makeHttpDatetimeAttribute(value)
            with self.assertRaises(Framework.github.BadAttributeException):
                actual.value

    def testMakeDatetimeAttributeBadValues(self):
        for value in ["not a timestamp", 1234]:
            actual = gho.GithubObject._makeDatetimeAttribute(value)

            self.assertEqual(gho._BadAttribute, type(actual))
            with self.assertRaises(Framework.github.BadAttributeException) as e:
                value = actual.value
            self.assertEqual(value, e.exception.actual_value)
            self.assertEqual(str, e.exception.expected_type)
            if isinstance(value, str):
                self.assertIsNotNone(e.exception.transformation_exception)
            else:
                self.assertIsNone(e.exception.transformation_exception)

    def testMakeTimestampAttribute(self):
        actual = gho.GithubObject._makeTimestampAttribute(None)
        self.assertEqual(gho._ValuedAttribute, type(actual))
        self.assertIsNone(actual.value)

        actual = gho.GithubObject._makeTimestampAttribute(1611405296)
        self.assertEqual(gho._ValuedAttribute, type(actual))
        self.assertEqual(datetime(2021, 1, 23, 12, 34, 56, tzinfo=timezone.utc), actual.value)

    def testMakeTimetsampAttributeBadValues(self):
        for value in ["1611405296", 1234.567]:
            actual = gho.GithubObject._makeTimestampAttribute(value)

            self.assertEqual(gho._BadAttribute, type(actual))
            with self.assertRaises(Framework.github.BadAttributeException) as e:
                value = actual.value
            self.assertEqual(value, e.exception.actual_value)
            self.assertEqual(int, e.exception.expected_type)
            self.assertIsNone(e.exception.transformation_exception)


class CompletableGithubObjectWithPaginatedProperty(Framework.TestCase):
    def testRepoCommitFilesDefault(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    commit = repo.get_commit("3253acaabd86de12b73d0a24c98eb9c13d1987b5")
                    files = list(commit.files)

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub"])
                    + ["/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1"],
                )

    def testRepoCommitFiles(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    commit = repo.get_commit("3253acaabd86de12b73d0a24c98eb9c13d1987b5")
                    files = list(commit.files)

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub"])
                    + [
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=2",
                        "/repositories/3544490/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=2&per_page=2",
                    ],
                )

    def testRepoCommitFilesWithPerPage(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    commit = repo.get_commit("3253acaabd86de12b73d0a24c98eb9c13d1987b5", commit_files_per_page=3)
                    files = list(commit.files)

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub"])
                    + [
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=3",
                        "/repositories/3544490/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=2&per_page=3",
                    ],
                )

    def testRepoCommitGetFilesDefault(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    commit = repo.get_commit("3253acaabd86de12b73d0a24c98eb9c13d1987b5")
                    files = list(commit.get_files())

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    (
                        []
                        if lazy
                        else [
                            "/repos/PyGithub/PyGithub",
                            "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1",
                        ]
                    )
                    + [
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1",
                    ],
                )

    def testRepoCommitGetFiles(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    commit = repo.get_commit("3253acaabd86de12b73d0a24c98eb9c13d1987b5")
                    files = list(commit.get_files())

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    (
                        []
                        if lazy
                        else [
                            "/repos/PyGithub/PyGithub",
                            "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=2",
                        ]
                    )
                    + [
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=2",
                        "/repositories/3544490/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=2&per_page=2",
                    ],
                )

    def testRepoCommitGetFilesWithPerPage(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    commit = repo.get_commit("3253acaabd86de12b73d0a24c98eb9c13d1987b5", commit_files_per_page=1)
                    files = list(commit.get_files(commit_files_per_page=3))

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    (
                        []
                        if lazy
                        else [
                            "/repos/PyGithub/PyGithub",
                            "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=1",
                        ]
                    )
                    + [
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=3",
                        "/repositories/3544490/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=2&per_page=3",
                    ],
                )

    def testRepoCommitsFiles(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    commits = repo.get_commits(sha="release-v2-0")
                    commit = commits[0]
                    files = list(commit.files)

                self.assertEqual(len(files), 3)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    ["github/Auth.py", "github/Requester.py", "tests/Authentication.py"],
                )

                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub"])
                    + [
                        "/repos/PyGithub/PyGithub/commits?per_page=2&sha=release-v2-0",
                        "/repos/PyGithub/PyGithub/commits/0791cc7b1a706ab5d7c607ddff35de4d486ba3e9?page=1&per_page=2",
                        "/repositories/3544490/commits/0791cc7b1a706ab5d7c607ddff35de4d486ba3e9?page=2&per_page=2",
                    ],
                )

    def testRepoComparisonCommitsFilesDefault(self):
        # replay data modified after record with
        # cat -n tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesDefault.txt | while read -r lineno line; do if [ $(( lineno % 11 )) -eq 10 ]; then jq . | sed -E -e 's/"patch":\s*".*[^\\]"/"patch":"…"/g' -e 's/"([^"]+_url)":\s*".*[^\\]"/"\1":"…"/g' | jq -c; else cat; fi <<< "$line"; done > tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesDefault.txt.bak
        # mv tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesDefault.txt.bak tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesDefault.txt
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    # tests paginated property of Comparison.commits and Commit.files
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    comparison = repo.compare(
                        "6cfe46b712e2bf65560bd8189c4654cd6c56eeca", "cef98416f45a9cdaf84d7f53cea13ac074a2c05d"
                    )
                    # PaginatedList commits should use default per_page
                    commits = list(comparison.commits)
                    self.assertEqual(len(commits), 7)
                    commit = commits[4]
                    self.assertEqual(commit.sha, "cbfe8d0f623ca29d984ec09d2b566e9ab10ae024")
                    # PaginatedList files should use default per_page
                    files = list(commit.files)

                self.assertEqual(len(files), 371)
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub"])
                    + [
                        "/repos/PyGithub/PyGithub/compare/6cfe46b712e2bf65560bd8189c4654cd6c56eeca...cef98416f45a9cdaf84d7f53cea13ac074a2c05d?page=1",
                        "/repos/PyGithub/PyGithub/commits/cbfe8d0f623ca29d984ec09d2b566e9ab10ae024?page=1",
                        "/repositories/3544490/commits/cbfe8d0f623ca29d984ec09d2b566e9ab10ae024?page=2",
                    ],
                )

    def testRepoComparisonCommitsFiles(self):
        # replay data modified after record with
        # cat -n tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFiles.txt | while read -r lineno line; do if [ $(( lineno % 11 )) -eq 10 ]; then jq . | sed -E -e 's/"patch":\s*".*[^\\]"/"patch":"…"/g' -e 's/"([^"]+_url)":\s*".*[^\\]"/"\1":"…"/g' | jq -c; else cat; fi <<< "$line"; done > tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFiles.txt.bak
        # mv tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFiles.txt.bak tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFiles.txt
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    # tests paginated property of Comparison.commits and Commit.files
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    comparison = repo.compare(
                        "19e1c5032397a95c58fe25760723ffc24cbe0ec8",
                        "4bf07a2f5123f78fc6759bc2ade0c74154c1ba86",
                    )
                    # PaginatedList commits should respect configured per_page
                    commits = list(comparison.commits)
                    self.assertEqual(len(commits), 4)
                    commit = commits[3]
                    self.assertEqual(commit.sha, "4bf07a2f5123f78fc6759bc2ade0c74154c1ba86")
                    # PaginatedList files should respect configured per_page
                    files = list(commit.files)

                self.assertEqual(len(files), 6)
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub"])
                    + [
                        "/repos/PyGithub/PyGithub/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=2",
                        "/repositories/3544490/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=2&per_page=2",
                        "/repos/PyGithub/PyGithub/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=2&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=3&per_page=2",
                    ],
                )

    def testRepoComparisonCommitsFilesWithPerPage(self):
        # replay data modified after record with
        # cat -n tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesWithPerPage.txt | while read -r lineno line; do if [ $(( lineno % 11 )) -eq 10 ]; then jq . | sed -E -e 's/"patch":\s*".*[^\\]"/"patch":"…"/g' -e 's/"([^"]+_url)":\s*".*[^\\]"/"\1":"…"/g' | jq -c; else cat; fi <<< "$line"; done > tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesWithPerPage.txt.bak
        # mv tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesWithPerPage.txt.bak tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesWithPerPage.txt
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    # tests paginated property of Comparison.commits and Commit.files
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    comparison = repo.compare(
                        "19e1c5032397a95c58fe25760723ffc24cbe0ec8",
                        "4bf07a2f5123f78fc6759bc2ade0c74154c1ba86",
                        comparison_commits_per_page=3,
                    )
                    # PaginatedList commits should use given per_page
                    commits = list(comparison.commits)
                    self.assertEqual(len(commits), 4)
                    commit = commits[3]
                    self.assertEqual(commit.sha, "4bf07a2f5123f78fc6759bc2ade0c74154c1ba86")
                    # PaginatedList files should respect configured per_page
                    files = list(commit.files)

                self.assertEqual(len(files), 6)
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub"])
                    + [
                        "/repos/PyGithub/PyGithub/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=3",
                        "/repositories/3544490/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=2&per_page=3",
                        "/repos/PyGithub/PyGithub/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=2&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=3&per_page=2",
                    ],
                )

    def testRepoComparisonCommitsFilesReversed(self):
        # replay data modified after record with
        # cat -n tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesReversed.txt | while read -r lineno line; do if [ $(( lineno % 11 )) -eq 10 ]; then jq . | sed -E -e 's/"patch":\s*".*[^\\]"/"patch":"…"/g' -e 's/"([^"]+_url)":\s*".*[^\\]"/"\1":"…"/g' | jq -c; else cat; fi <<< "$line"; done > tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesReversed.txt.bak
        # mv tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesReversed.txt.bak tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesReversed.txt
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    # tests paginated property of Comparison.commits and Commit.files
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    comparison = repo.compare(
                        "19e1c5032397a95c58fe25760723ffc24cbe0ec8",
                        "4bf07a2f5123f78fc6759bc2ade0c74154c1ba86",
                    )
                    # PaginatedList commits should respect configured per_page
                    commits = list(reversed(comparison.commits))
                    self.assertEqual(len(commits), 4)
                    commit = commits[0]
                    self.assertEqual(commit.sha, "4bf07a2f5123f78fc6759bc2ade0c74154c1ba86")
                    # PaginatedList files should respect configured per_page
                    files = list(reversed(commit.files))

                self.assertEqual(len(files), 6)
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub"])
                    + [
                        "/repos/PyGithub/PyGithub/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=2",
                        "/repositories/3544490/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=2&per_page=2",
                        "/repositories/3544490/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=2",
                        "/repos/PyGithub/PyGithub/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=3&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=2&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=2",
                    ],
                )

    def testRepoComparisonCommitsFilesReversedWithPerPage(self):
        # replay data modified after record with
        # cat -n tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesReversedWithPerPage.txt | while read -r lineno line; do if [ $(( lineno % 11 )) -eq 10 ]; then jq . | sed -E -e 's/"patch":\s*".*[^\\]"/"patch":"…"/g' -e 's/"([^"]+_url)":\s*".*[^\\]"/"\1":"…"/g' | jq -c; else cat; fi <<< "$line"; done > tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesReversedWithPerPage.txt.bak
        # mv tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesReversedWithPerPage.txt.bak tests/ReplayData/CompletableGithubObjectWithPaginatedProperty.testRepoComparisonCommitsFilesReversedWithPerPage.txt
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    # tests paginated property of Comparison.commits and Commit.files
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    comparison = repo.compare(
                        "19e1c5032397a95c58fe25760723ffc24cbe0ec8",
                        "4bf07a2f5123f78fc6759bc2ade0c74154c1ba86",
                        comparison_commits_per_page=3,
                    )
                    # PaginatedList commits should use given per_page
                    commits = list(reversed(comparison.commits))
                    self.assertEqual(len(commits), 4)
                    commit = commits[0]
                    self.assertEqual(commit.sha, "4bf07a2f5123f78fc6759bc2ade0c74154c1ba86")
                    # PaginatedList files should respect configured per_page
                    files = list(reversed(commit.files))

                self.assertEqual(len(files), 6)
                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub"])
                    + [
                        "/repos/PyGithub/PyGithub/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=3",
                        "/repositories/3544490/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=2&per_page=3",
                        "/repositories/3544490/compare/19e1c5032397a95c58fe25760723ffc24cbe0ec8...4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=3",
                        "/repos/PyGithub/PyGithub/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=3&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=2&per_page=2",
                        "/repositories/3544490/commits/4bf07a2f5123f78fc6759bc2ade0c74154c1ba86?page=1&per_page=2",
                    ],
                )

    def testPullCommitsFilesDefault(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    pull = repo.get_pull(3370)
                    # PaginatedList commits should use default per_page
                    commits = list(pull.get_commits())
                    self.assertEqual(len(commits), 1)
                    commit = commits[0]
                    # PaginatedList files should use default per_page
                    files = list(commit.files)

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )

                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub", "/repos/PyGithub/PyGithub/pulls/3370"])
                    + [
                        "/repos/PyGithub/PyGithub/pulls/3370/commits",
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1",
                    ],
                )

    def testPullCommitsFiles(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    pull = repo.get_pull(3370)
                    # PaginatedList commits should respect configured per_page
                    commits = list(pull.get_commits())
                    self.assertEqual(len(commits), 1)
                    commit = commits[0]
                    # PaginatedList files should respect configured per_page
                    files = list(commit.files)

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )

                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub", "/repos/PyGithub/PyGithub/pulls/3370"])
                    + [
                        "/repos/PyGithub/PyGithub/pulls/3370/commits?per_page=2",
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=2",
                        "/repositories/3544490/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=2&per_page=2",
                    ],
                )

    def testPullCommitsGetFilesDefault(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    pull = repo.get_pull(3370)
                    # PaginatedList commits should respect configured per_page
                    commits = list(pull.get_commits())
                    self.assertEqual(len(commits), 1)
                    commit = commits[0]
                    # PaginatedList commits should use given per_page
                    files = list(commit.get_files(commit_files_per_page=100))

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )

                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub", "/repos/PyGithub/PyGithub/pulls/3370"])
                    + [
                        "/repos/PyGithub/PyGithub/pulls/3370/commits",
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=100",
                    ],
                )

    def testPullCommitsGetFiles(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    pull = repo.get_pull(3370)
                    # PaginatedList commits should respect configured per_page
                    commits = list(pull.get_commits())
                    self.assertEqual(len(commits), 1)
                    commit = commits[0]
                    # PaginatedList commits should respect configured per_page
                    files = list(commit.get_files())

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )

                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub", "/repos/PyGithub/PyGithub/pulls/3370"])
                    + [
                        "/repos/PyGithub/PyGithub/pulls/3370/commits?per_page=2",
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=2",
                        "/repositories/3544490/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=2&per_page=2",
                    ],
                )

    def testPullCommitsGetFilesWithPerPage(self):
        for lazy in [True, False]:
            with self.subTest(lazy=lazy):
                with self.captureRequests() as requests:
                    self.g.per_page = 2
                    repo = self.g.get_repo("PyGithub/PyGithub", lazy=lazy)
                    pull = repo.get_pull(3370)
                    # PaginatedList commits should respect configured per_page
                    commits = list(pull.get_commits())
                    self.assertEqual(len(commits), 1)
                    commit = commits[0]
                    # PaginatedList commits should use given per_page
                    files = list(commit.get_files(commit_files_per_page=100))

                self.assertEqual(len(files), 4)
                self.assertListKeyEqual(
                    files,
                    lambda f: f.filename,
                    [
                        ".github/workflows/_build-pkg.yml",
                        ".github/workflows/ci.yml",
                        ".github/workflows/lint.yml",
                        ".github/workflows/openapi.yml",
                    ],
                )

                self.assertListKeyEqual(
                    requests,
                    lambda r: r.url,
                    ([] if lazy else ["/repos/PyGithub/PyGithub", "/repos/PyGithub/PyGithub/pulls/3370"])
                    + [
                        "/repos/PyGithub/PyGithub/pulls/3370/commits?per_page=2",
                        "/repos/PyGithub/PyGithub/commits/3253acaabd86de12b73d0a24c98eb9c13d1987b5?page=1&per_page=100",
                    ],
                )


class TestingClass(gho.NonCompletableGithubObject):
    def _initAttributes(self) -> None:
        pass

    def _useAttributes(self, attributes: Any) -> None:
        pass



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class License(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.license = self.g.get_license("mit")

    def testAttributes(self):
        self.assertEqual(self.license.html_url, "http://choosealicense.com/licenses/mit/")
        self.assertEqual(self.license.key, "mit")
        self.assertEqual(self.license.name, "MIT License")
        self.assertEqual(
            self.license.description,
            "A short and simple permissive license with conditions only requiring preservation of copyright and license notices. Licensed works, modifications, and larger works may be distributed under different terms and without source code.",
        )
        self.assertEqual(self.license.node_id, "MDc6TGljZW5zZTEz")
        self.assertEqual(self.license.spdx_id, "MIT")
        self.assertEqual(
            self.license.body,
            """MIT License\n\nCopyright (c) [year] [fullname]\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.\n""",
        )
        self.assertEqual(
            self.license.permissions,
            ["commercial-use", "modifications", "distribution", "private-use"],
        )
        self.assertEqual(self.license.conditions, ["include-copyright"])
        self.assertEqual(self.license.limitations, ["liability", "warranty"])
        self.assertEqual(self.license.url, "https://api.github.com/licenses/mit")
        self.assertEqual(self.license.html_url, "http://choosealicense.com/licenses/mit/")
        self.assertEqual(
            self.license.implementation,
            """Create a text file (typically named LICENSE or LICENSE.txt) in the root of your source code and copy the text of the license into the file. Replace [year] with the current year and [fullname] with the name (or names) of the copyright holders.""",
        )
        self.assertEqual(repr(self.license), 'License(name="MIT License")')



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2016 mattjmorrison <mattjmorrison@mattjmorrison.com>               #
# Copyright 2018 Isuru Fernando <isuruf@gmail.com>                             #
# Copyright 2018 Jacopo Notarstefano <jacopo.notarstefano@gmail.com>           #
# Copyright 2018 James D'Amato <james.j.damato@gmail.com>                      #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 Tim Boring <tboring@hearst.com>                               #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Adam Baratz <adam.baratz@gmail.com>                           #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Adrian Bridgett <58699309+tl-adrian-bridgett@users.noreply.github.com>#
# Copyright 2020 Andy Grunwald <andygrunwald@gmail.com>                        #
# Copyright 2020 Gilad Shefer <giladshefer@gmail.com>                          #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2020 Tal Machani <12785464+talmachani@users.noreply.github.com>    #
# Copyright 2021 秋葉 <ambiguous404@gmail.com>                                   #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Andrii Kezikov <cheshirez@gmail.com>                          #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################


from __future__ import annotations

import warnings
from datetime import datetime, timezone
from typing import TYPE_CHECKING

from . import Framework

if TYPE_CHECKING:
    from github.Repository import Repository


class Team(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.org = self.g.get_organization("BeaverSoftware")
        self.team = self.org.get_team(12345678)

    def testAttributes(self):
        self.assertEqual(self.team.created_at, datetime(2024, 6, 18, 10, 27, 23, tzinfo=timezone.utc))
        self.assertEqual(self.team.description, "a team")
        self.assertIsNone(self.team.enterprise_id)
        self.assertIsNone(self.team.group_id)
        self.assertIsNone(self.team.group_name)
        self.assertEqual(self.team.html_url, "https://github.com/orgs/BeaverSoftware/teams/team-slug")
        self.assertEqual(self.team.id, 12345678)
        self.assertIsNone(self.team.ldap_dn)
        self.assertEqual(self.team.members_count, 1)
        self.assertEqual(
            self.team.members_url, "https://api.github.com/organizations/1234567/team/12345678/members{/member}"
        )
        self.assertEqual(self.team.name, "Team")
        self.assertEqual(self.team.node_id, "AbCdEfG")
        self.assertEqual(self.team.notification_setting, "notifications_disabled")
        self.assertEqual(self.team.organization.login, "BeaverSoftware")
        self.assertIsNone(self.team.organization_id)
        self.assertIsNone(self.team.organization_selection_type)
        self.assertIsNone(self.team.parent)
        self.assertEqual(self.team.permission, "pull")
        self.assertIsNone(self.team.permissions)
        self.assertEqual(self.team.privacy, "closed")
        self.assertEqual(self.team.repos_count, 0)
        self.assertEqual(self.team.repositories_url, "https://api.github.com/organizations/1234567/team/12345678/repos")
        self.assertEqual(self.team.slug, "team-slug")
        self.assertIsNone(self.team.sync_to_organizations)
        self.assertIsNone(self.team.type)
        self.assertEqual(self.team.updated_at, datetime(2024, 6, 18, 10, 27, 23, tzinfo=timezone.utc))
        self.assertEqual(self.team.url, "https://api.github.com/organizations/1234567/team/12345678")
        self.assertEqual(self.team.organization, self.org)
        self.assertEqual(self.team.privacy, "closed")
        self.assertEqual(self.team.parent, None)
        self.assertEqual(repr(self.team), 'Team(name="Team", id=12345678)')
        self.assertEqual(self.team.html_url, "https://github.com/orgs/BeaverSoftware/teams/team-slug")

    def testLazyAttributes(self):
        team = self.g.withLazy(True).get_organization("org").get_team(42)
        self.assertEqual(str(team), "Team(name=None, id=42)")
        self.assertEqual(team._identity, 42)
        self.assertEqual(team.id, 42)
        self.assertEqual(team.url, "/teams/42")

    def testDiscussions(self):
        discussions = list(self.team.get_discussions())
        self.assertEqual(len(discussions), 1)

        d = discussions[0]
        self.assertEqual(d.author.login, "jacquev6")
        self.assertEqual(d.body, "BODY")
        self.assertEqual(d.body_html, "<p>BODY</p>")
        self.assertEqual(d.body_version, "bedf0740b01d2d758cff9873c2387817")
        self.assertEqual(d.comments_count, 0)
        self.assertEqual(
            d.comments_url, "https://api.github.com/organizations/1234567/team/12345678/discussions/1/comments"
        )
        self.assertEqual(d.created_at, datetime(2019, 10, 8, 21, 3, 36, tzinfo=timezone.utc))
        self.assertEqual(
            d.html_url,
            "https://github.com/orgs/BeaverSoftware/teams/Team/discussions/1",
        )
        self.assertEqual(d.last_edited_at, None)
        self.assertEqual(d.node_id, "MDE0OlRlYW1EaXNjdXNzaW9uMzA=")
        self.assertEqual(d.number, 1)
        self.assertEqual(d.pinned, True)
        self.assertEqual(d.private, False)
        self.assertEqual(d.team_url, "https://api.github.com/organizations/1234567/team/12345678")
        self.assertEqual(d.title, "TITLE")
        self.assertEqual(d.updated_at, datetime(2019, 10, 8, 21, 3, 36, tzinfo=timezone.utc))
        self.assertEqual(d.url, "https://api.github.com/organizations/1234567/team/12345678/discussions/1")
        self.assertEqual(repr(d), 'TeamDiscussion(title="TITLE", number=1)')

    def testMembers(self):
        user = self.g.get_user("jacquev6")
        self.assertListKeyEqual(self.team.get_members(), None, [])
        self.assertFalse(self.team.has_in_members(user))
        self.team.add_membership(user)
        self.assertListKeyEqual(self.team.get_members(), lambda u: u.login, ["jacquev6"])
        self.assertTrue(self.team.has_in_members(user))
        self.team.remove_membership(user)
        self.assertListKeyEqual(self.team.get_members(), None, [])
        self.assertFalse(self.team.has_in_members(user))
        self.team.add_membership(user, "maintainer")
        self.assertRaises(AssertionError, self.team.add_membership, user, "admin")
        self.team.remove_membership(user)

    def testTeamMembership(self):
        user = self.g.get_user("jacquev6")
        self.assertEqual(list(self.team.get_members()), [])
        self.assertFalse(self.team.has_in_members(user))
        self.team.add_membership(user)
        self.assertListKeyEqual(self.team.get_members(), lambda u: u.login, ["jacquev6"])
        self.assertTrue(self.team.has_in_members(user))
        membership_data = self.team.get_team_membership(user)
        self.assertEqual(membership_data.user.login, "jacquev6")
        self.assertEqual(membership_data.role, "member")
        self.assertEqual(membership_data.organization.login, "BeaverSoftware")

    def testRepoPermission(self):
        repo = self.org.get_repo("FatherBeaver")
        # Ignore the warning since this method is deprecated
        warnings.filterwarnings("ignore", category=DeprecationWarning)
        self.team.set_repo_permission(repo, "admin")
        warnings.resetwarnings()

    def testUpdateTeamRepository(self):
        repo = self.org.get_repo("FatherBeaver")
        self.assertTrue(self.team.update_team_repository(repo, "admin"))

    def doTestRepos(self, repo: str | Repository):
        self.assertListKeyEqual(self.team.get_repos(), None, [])
        self.assertFalse(self.team.has_in_repos(repo))
        self.assertIsNone(self.team.get_repo_permission(repo))
        self.team.add_to_repos(repo)
        self.assertListKeyEqual(self.team.get_repos(), lambda r: r.name, ["FatherBeaver"])
        self.assertTrue(self.team.has_in_repos(repo))
        permissions = self.team.get_repo_permission(repo)
        self.assertTrue(permissions.pull)
        self.team.remove_from_repos(repo)
        self.assertListKeyEqual(self.team.get_repos(), None, [])
        self.assertFalse(self.team.has_in_repos(repo))

    def testRepos(self):
        self.doTestRepos(self.org.get_repo("FatherBeaver"))

    def testReposStr(self):
        with self.replayData("Team.testRepos.txt"):
            self.doTestRepos(self.org.get_repo("FatherBeaver")._identity)

    def testEditWithoutArguments(self):
        self.team.edit("Name edited by PyGithub")
        self.assertEqual(self.team.name, "Name edited by PyGithub")

    def testEditWithAllArguments(self):
        parent = self.org.get_team(141496)
        self.team.edit(
            "Name edited twice by PyGithub",
            "Description edited by PyGithub",
            "admin",
            "secret",
            parent.id,
            "notifications_disabled",
        )
        self.assertEqual(self.team.name, "Name edited twice by PyGithub")
        self.assertEqual(self.team.description, "Description edited by PyGithub")
        self.assertEqual(self.team.permission, "admin")
        self.assertEqual(self.team.privacy, "secret")
        self.assertEqual(self.team.parent, parent)
        self.assertEqual(self.team.notification_setting, "notifications_disabled")

    def testGetTeams(self):
        nested_teams = self.team.get_teams()
        self.assertListKeyEqual(nested_teams, lambda t: t.name, ["DummyTeam1", "DummyTeam2", "DummyTeam3"])
        parent = nested_teams[0].parent
        self.assertEqual(self.team.name, parent.name)
        self.assertEqual(self.team.id, parent.id)

    def testDelete(self):
        self.team.delete()



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Mateusz Loskot <mateusz@loskot.net>                           #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class Label(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.label = self.g.get_repo("PyGithub/PyGithub").get_label("Bug")

    def testAttributes(self):
        self.assertEqual(self.label.color, "e10c02")
        self.assertEqual(self.label.default, True)
        self.assertIsNone(self.label.description)
        self.assertEqual(self.label.id, 3330121)
        self.assertEqual(self.label.name, "bug")
        self.assertIsNone(self.label.description)
        self.assertEqual(self.label.node_id, "MDU6TGFiZWwzMzMwMTIx")
        self.assertEqual(self.label.url, "https://api.github.com/repos/PyGithub/PyGithub/labels/bug")
        self.assertEqual(repr(self.label), 'Label(name="bug")')

    def testLazyAttributes(self):
        label = self.g.withLazy(True).get_repo("lazy/repo").get_label("good first issue")
        self.assertEqual(str(label), 'Label(name="good first issue")')
        self.assertEqual(label._identity, "good%20first%20issue")
        self.assertEqual(label.name, "good first issue")
        self.assertEqual(label.url, "/repos/lazy/repo/labels/good%20first%20issue")

    def testEdit(self):
        self.label.edit("LabelEditedByPyGithub", "0000ff", "Description of LabelEditedByPyGithub")
        self.assertEqual(self.label.color, "0000ff")
        self.assertEqual(self.label.description, "Description of LabelEditedByPyGithub")
        self.assertEqual(self.label.name, "LabelEditedByPyGithub")
        self.assertEqual(
            self.label.url,
            "https://api.github.com/repos/PyGithub/PyGithub/labels/LabelEditedByPyGithub",
        )

    def testDelete(self):
        self.label.delete()



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2026 Matt Davis <35502728+matt-davis27@users.noreply.github.com>   #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import github.OrganizationCodeScanAlert
import github.PaginatedList
import github.Repository

from . import Framework


class OrganizationCodeScanAlert(Framework.TestCase):
    alert: github.OrganizationCodeScanAlert.OrganizationCodeScanAlert

    def setUp(self):
        super().setUp()
        self.org = self.g.get_organization(
            "github"
        )  # This is a placeholder organization that should always exist, switch this to your own test organization

    def testMultipleAlerts(self):
        multiple_alerts = self.org.get_codescan_alerts()
        self.assertIsInstance(multiple_alerts, github.PaginatedList.PaginatedList)
        self.assertIsInstance(multiple_alerts[0], github.OrganizationCodeScanAlert.OrganizationCodeScanAlert)
        alert_list = [alert for alert in multiple_alerts]
        self.assertEqual(len(alert_list), 2)  # Update this when more alerts are added

        test_alert = alert_list[0]
        self.assertEqual(test_alert.number, 4)
        self.assertEqual(test_alert.created_at, datetime(2020, 2, 13, 12, 29, 18, tzinfo=timezone.utc))
        self.assertEqual(test_alert.updated_at, datetime(2025, 8, 25, 16, 3, 10, tzinfo=timezone.utc))
        self.assertEqual(test_alert.url, "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/4")
        self.assertEqual(test_alert.html_url, "https://github.com/octocat/hello-world/code-scanning/4")
        self.assertEqual(test_alert.state, "open")
        self.assertIsNone(test_alert.dismissed_by)
        self.assertIsNone(test_alert.dismissed_at)
        self.assertIsNone(test_alert.dismissed_reason)
        self.assertIsNone(test_alert.dismissed_comment)
        self.assertEqual(test_alert.rule.id, "js/zipslip")
        self.assertEqual(test_alert.rule.severity, "error")
        self.assertEqual(test_alert.rule.description, "Arbitrary file write during zip extraction")
        self.assertEqual(test_alert.rule.name, "js/zipslip")
        self.assertEqual(test_alert.rule.tags[0], "security")
        self.assertEqual(test_alert.rule.tags[1], "external/cwe/cwe-022")
        self.assertEqual(test_alert.tool.name, "CodeQL")
        self.assertIsNone(test_alert.tool.guid)
        self.assertEqual(test_alert.tool.version, "2.4.0")
        self.assertEqual(test_alert.most_recent_instance.ref, "refs/heads/main")
        self.assertEqual(
            test_alert.most_recent_instance.analysis_key, ".github/workflows/codeql-analysis.yml:CodeQL-Build"
        )
        self.assertEqual(test_alert.most_recent_instance.environment, "{}")
        self.assertEqual(test_alert.most_recent_instance.category, ".github/workflows/codeql-analysis.yml:CodeQL-Build")
        self.assertEqual(test_alert.most_recent_instance.state, "open")
        self.assertEqual(test_alert.most_recent_instance.commit_sha, "39406e42cb832f683daa691dd652a8dc36ee8930")
        self.assertEqual(test_alert.most_recent_instance.message["text"], "This path depends on a user-provided value.")
        self.assertEqual(test_alert.most_recent_instance.location.path, "spec-main/api-session-spec.ts")
        self.assertEqual(test_alert.most_recent_instance.location.start_line, 917)
        self.assertEqual(test_alert.most_recent_instance.location.end_line, 917)
        self.assertEqual(test_alert.most_recent_instance.location.start_column, 7)
        self.assertEqual(test_alert.most_recent_instance.location.end_column, 18)
        self.assertEqual(test_alert.most_recent_instance.classifications[0], "test")
        self.assertEqual(
            test_alert.instances_url,
            "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/4/instances",
        )
        self.assertIsNone(test_alert.dismissal_approved_by)
        self.assertIsInstance(test_alert.repository, github.Repository.Repository)
        self.assertEqual(test_alert.repository.name, "Hello-World")
        self.assertEqual(test_alert.repository.full_name, "octocat/Hello-World")

    def testRepr(self):
        multiple_alerts = self.org.get_codescan_alerts()
        alert_list = [alert for alert in multiple_alerts]
        test_alert = alert_list[0]
        self.assertEqual(repr(test_alert), 'OrganizationCodeScanAlert(number=4, id="js/zipslip")')

    def testGetAlertsWithArguments(self):
        alerts = self.org.get_codescan_alerts(
            tool_name="CodeQL", ref="refs/heads/main", sort="created", direction="asc", state="open", severity="error"
        )
        # Note, this doesn't test "tool_guid" or "pr" arguments as they are not found in the test data
        self.assertEqual(len(list(alerts)), 2)  # Update this when more alerts are added



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2020 Yannick Jadoul <yannick.jadoul@belgacom.net>                  #
# Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jeppe Fihl-Pearson <tenzer@tenzer.dk>                         #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Sasha Chung <50770626+nuang-ee@users.noreply.github.com>      #
# Copyright 2024 Chris Gavin <chris@chrisgavin.me>                             #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Geoffrey <geoffrey@moveparallel.com>                          #
# Copyright 2025 Alejandro Perez Gancedo <37455131+LifeLex@users.noreply.github.com>#
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class WorkflowRun(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("PyGithub/PyGithub")
        self.workflow_run = self.repo.get_workflow_run(3881497935)

    def testAttributes(self):
        self.assertEqual(
            repr(self.workflow_run),
            'WorkflowRun(url="https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935", id=3881497935)',
        )
        self.assertEqual(self.workflow_run.actor.login, "nuang-ee")
        self.assertEqual(
            self.workflow_run.artifacts_url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/artifacts",
        )
        self.assertEqual(
            self.workflow_run.cancel_url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/cancel",
        )
        self.assertEqual(self.workflow_run.check_suite_id, 10279069747)
        self.assertEqual(self.workflow_run.check_suite_node_id, "CS_kwDOADYVqs8AAAACZK4oMw")
        self.assertEqual(
            self.workflow_run.check_suite_url, "https://api.github.com/repos/PyGithub/PyGithub/check-suites/10279069747"
        )
        self.assertEqual(self.workflow_run.conclusion, "success")
        self.assertEqual(self.workflow_run.created_at, datetime(2023, 1, 10, 8, 24, 19, tzinfo=timezone.utc))
        self.assertEqual(self.workflow_run.display_title, "TEST PR")
        self.assertEqual(self.workflow_run.event, "pull_request")
        self.assertEqual(self.workflow_run.head_branch, "feat/workflow-run")
        self.assertEqual(self.workflow_run.head_commit.sha, "c6e5cac67a58a4eb11f1f28567a77a6e2cc8ee98")
        self.assertEqual(self.workflow_run.head_repository.full_name, "nuang-ee/PyGithub")
        self.assertIsNone(self.workflow_run.head_repository_id)
        self.assertEqual(self.workflow_run.head_sha, "c6e5cac67a58a4eb11f1f28567a77a6e2cc8ee98")
        self.assertEqual(self.workflow_run.html_url, "https://github.com/PyGithub/PyGithub/actions/runs/3881497935")
        self.assertEqual(self.workflow_run.id, 3881497935)
        self.assertEqual(
            self.workflow_run.jobs_url, "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/jobs"
        )
        self.assertEqual(
            self.workflow_run.logs_url, "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/logs"
        )
        self.assertEqual(self.workflow_run.name, "CI")
        self.assertEqual(self.workflow_run.head_branch, "feat/workflow-run")
        self.assertEqual(self.workflow_run.head_sha, "c6e5cac67a58a4eb11f1f28567a77a6e2cc8ee98")
        self.assertEqual(self.workflow_run.node_id, "WFR_kwLOADYVqs7nWvVP")
        self.assertEqual(self.workflow_run.path, ".github/workflows/ci.yml")
        self.assertEqual(self.workflow_run.display_title, "TEST PR")
        self.assertIsNone(self.workflow_run.previous_attempt_url)
        self.assertEqual(len(self.workflow_run.pull_requests), 0)
        self.assertEqual(self.workflow_run.referenced_workflows, [])
        self.assertEqual(self.workflow_run.repository.full_name, "PyGithub/PyGithub")
        self.assertIsNone(self.workflow_run.repository_id)
        self.assertEqual(
            self.workflow_run.rerun_url, "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/rerun"
        )
        self.assertEqual(self.workflow_run.run_attempt, 1)
        self.assertEqual(self.workflow_run.run_number, 930)
        self.assertEqual(self.workflow_run.run_attempt, 1)
        self.assertEqual(self.workflow_run.run_started_at, datetime(2023, 1, 10, 8, 24, 19, tzinfo=timezone.utc))
        self.assertEqual(self.workflow_run.event, "pull_request")
        self.assertEqual(self.workflow_run.status, "completed")
        self.assertEqual(self.workflow_run.conclusion, "success")
        self.assertEqual(self.workflow_run.triggering_actor.login, "nuang-ee")
        self.assertEqual(self.workflow_run.updated_at, datetime(2023, 1, 10, 8, 28, 20, tzinfo=timezone.utc))
        self.assertEqual(
            self.workflow_run.url, "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935"
        )
        self.assertEqual(self.workflow_run.workflow_id, 1903133)
        self.assertEqual(
            self.workflow_run.url, "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935"
        )
        self.assertEqual(self.workflow_run.html_url, "https://github.com/PyGithub/PyGithub/actions/runs/3881497935")
        self.assertEqual(self.workflow_run.pull_requests, [])
        created_at = datetime(2023, 1, 10, 8, 24, 19, tzinfo=timezone.utc)
        self.assertEqual(self.workflow_run.created_at, created_at)
        updated_at = datetime(2023, 1, 10, 8, 28, 20, tzinfo=timezone.utc)
        self.assertEqual(self.workflow_run.updated_at, updated_at)
        self.assertEqual(
            self.workflow_run.jobs_url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/jobs",
        )
        self.assertEqual(
            self.workflow_run.logs_url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/logs",
        )
        self.assertEqual(
            self.workflow_run.check_suite_url,
            "https://api.github.com/repos/PyGithub/PyGithub/check-suites/10279069747",
        )
        self.assertEqual(
            self.workflow_run.artifacts_url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/artifacts",
        )
        self.assertEqual(
            self.workflow_run.cancel_url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/cancel",
        )
        self.assertEqual(
            self.workflow_run.rerun_url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/3881497935/rerun",
        )
        self.assertEqual(
            self.workflow_run.workflow_url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/workflows/1903133",
        )
        self.assertEqual(self.workflow_run.head_commit.message, "add attribute 'name' on WorkflowRun")
        self.assertEqual(self.workflow_run.head_commit.tree.sha, "3ce398f9ee2571549b7fea545bfa5bf28e3ca0f5")
        self.assertEqual(self.workflow_run.repository.name, "PyGithub")
        self.assertEqual(self.workflow_run.head_repository.name, "PyGithub")

    def test_timing(self):
        timing = self.workflow_run.timing()
        self.assertEqual(
            timing.billable,
            {
                "UBUNTU": {
                    "job_runs": [
                        {"duration_ms": 0, "job_id": 10545727758},
                        {"duration_ms": 0, "job_id": 10545727888},
                        {"duration_ms": 0, "job_id": 10545728039},
                        {"duration_ms": 0, "job_id": 10545728190},
                        {"duration_ms": 0, "job_id": 10545728356},
                    ],
                    "jobs": 5,
                    "total_ms": 0,
                }
            },
        )
        self.assertEqual(timing.run_duration_ms, 241000)

    def test_timing_no_run_duration(self):
        timing = self.workflow_run.timing()
        self.assertEqual(timing.billable, {})
        self.assertIsNone(timing.run_duration_ms)

    def test_rerun(self):
        wr = self.repo.get_workflow_run(3910280793)
        self.assertFalse(wr.rerun())

    def test_rerun_failed_jobs(self):
        wr = self.repo.get_workflow_run(3881497935)
        self.assertTrue(wr.rerun_failed_jobs())

    def test_rerun_with_successful_run(self):
        wr = self.repo.get_workflow_run(3881497935)
        self.assertFalse(wr.rerun())

    def test_cancel(self):
        wr = self.repo.get_workflow_run(3911660493)
        self.assertFalse(wr.cancel())

    def test_delete(self):
        wr = self.repo.get_workflow_run(3881497935)
        self.assertFalse(wr.delete())

    def test_jobs(self):
        self.assertListKeyEqual(
            self.workflow_run.jobs(),
            lambda j: j.id,
            [10545727758, 10545727888, 10545728039, 10545728190, 10545728356],
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Alice GIRARD <bouhahah@gmail.com>                             #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 Chris Keating <christopherkeating@gmail.com>                  #
# Copyright 2021 MeggyCal <MeggyCal@users.noreply.github.com>                  #
# Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2026 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class PublicKey(Framework.TestCase):
    def testAttributes(self):
        self.public_key = self.g.get_user().get_repo("PyGithub").get_public_key()
        self.assertIsNone(self.public_key.created_at)
        self.assertIsNone(self.public_key.id)
        self.assertEqual(self.public_key.key, "u5e1Z25+z8pmgVVt5Pd8k0z/sKpVL1MXYtRAecE4vm8=")
        self.assertEqual(self.public_key.key_id, "568250167242549743")
        self.assertEqual(
            repr(self.public_key),
            'PublicKey(key_id="568250167242549743", key="u5e1Z25+z8pmgVVt5Pd8k0z/sKpVL1MXYtRAecE4vm8=")',
        )
        self.assertIsNone(self.public_key.title)
        self.assertEqual(
            self.public_key.url, "https://api.github.com/repos/jacquev6/PyGithub/actions/secrets/public-key"
        )

    def testAttributes_with_int_key_id(self):
        self.public_key = self.g.get_user().get_repo("PyGithub").get_public_key()
        self.assertEqual(self.public_key.key, "u5e1Z25+z8pmgVVt5Pd8k0z/sKpVL1MXYtRAecE4vm8=")
        self.assertEqual(self.public_key.key_id, 568250167242549743)
        self.assertEqual(
            repr(self.public_key),
            'PublicKey(key_id=568250167242549743, key="u5e1Z25+z8pmgVVt5Pd8k0z/sKpVL1MXYtRAecE4vm8=")',
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Tim Gates <tim.gates@iress.com>                               #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class GitCommit(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.commit = self.g.get_repo("PyGithub/PyGithub").get_git_commit("3d84a47a88f6757514cb3ee91b829f53ba09e7e0")

    def testAttributes(self):
        self.assertEqual(self.commit.author.name, "Enrico Minack")
        self.assertEqual(self.commit.author.email, "github@enrico.minack.dev")
        self.assertEqual(self.commit.author.date, datetime(2024, 12, 18, 10, 40, 19, tzinfo=timezone.utc))
        self.assertIsNone(self.commit.comment_count)
        self.assertEqual(self.commit.committer.name, "GitHub")
        self.assertEqual(self.commit.committer.email, "noreply@github.com")
        self.assertEqual(self.commit.committer.date, datetime(2024, 12, 18, 10, 40, 19, tzinfo=timezone.utc))
        self.assertEqual(
            self.commit.html_url, "https://github.com/PyGithub/PyGithub/commit/3d84a47a88f6757514cb3ee91b829f53ba09e7e0"
        )
        self.assertIsNone(self.commit.id)
        self.assertEqual(
            self.commit.message,
            "Get branches where commit is head (#3083)\n\nImplements `GET\r\n/repos/{owner}/{repo}/commits/{commit_sha}/branches-where-head`\r\n\r\nhttps://docs.github.com/rest/commits/commits#list-branches-for-head-commit",
        )
        self.assertEqual(self.commit.node_id, "C_kwDOADYVqtoAKDNkODRhNDdhODhmNjc1NzUxNGNiM2VlOTFiODI5ZjUzYmEwOWU3ZTA")
        self.assertEqual(len(self.commit.parents), 1)
        self.assertEqual(self.commit.parents[0].sha, "a50ae51b2c351b889055568bcaa9ab6000f1677f")
        self.assertEqual(self.commit.sha, "3d84a47a88f6757514cb3ee91b829f53ba09e7e0")
        self.assertIsNone(self.commit.timestamp)
        self.assertEqual(self.commit.tree.sha, "d9e2468f2db35e158eb65e91b249dde20ca88c86")
        self.assertIsNone(self.commit.tree_id)
        self.assertEqual(
            self.commit.url,
            "https://api.github.com/repos/PyGithub/PyGithub/git/commits/3d84a47a88f6757514cb3ee91b829f53ba09e7e0",
        )
        self.assertEqual(repr(self.commit), 'GitCommit(sha="3d84a47a88f6757514cb3ee91b829f53ba09e7e0")')
        self.assertEqual(repr(self.commit.author), 'GitAuthor(name="Enrico Minack")')
        self.assertEqual(
            self.commit.verification.payload,
            "tree d9e2468f2db35e158eb65e91b249dde20ca88c86\n"
            "parent a50ae51b2c351b889055568bcaa9ab6000f1677f\n"
            "author Enrico Minack <github@enrico.minack.dev> 1734518419 +0100\n"
            "committer GitHub <noreply@github.com> 1734518419 +0100\n"
            "\n"
            "Get branches where commit is head (#3083)\n"
            "\n"
            "Implements `GET\r\n"
            "/repos/{owner}/{repo}/commits/{commit_sha}/branches-where-head`\r\n"
            "\r\n"
            "https://docs.github.com/rest/commits/commits#list-branches-for-head-commit",
        )
        self.assertEqual(self.commit.verification.reason, "valid")
        self.assertEqual(
            self.commit.verification.signature,
            "-----BEGIN PGP SIGNATURE-----\n"
            "\n"
            "wsFcBAABCAAQBQJnYqaTCRC1aQ7uu5UhlAAAp3wQAGPaBPEC4WlL9rvDA6G+kaRZ\n"
            "+296Qb/jRjZX6joS//aQq5rjHKn//qTv13lTBQnM1a7gFfvA7TSoxNsoqzkk6DAe\n"
            "ME7qoSMYWl+GdvXsySo4ksGbFN0LL77CSmJyFXRXB5TIjUJT7dbjTkjE9/4zcfq4\n"
            "mR8D57GowX7YgqUeRzf8+5zz7ySJ9hAMcF/n+OJjLiew0RFp3hQBSFOr/1B4YJbL\n"
            "0Ln9i/DH9KBhwIUnc68k04GxVtAMaS7X0SOVbezylaBlQyF2JV3bDbb38h77KPJ1\n"
            "ln0qPi+hamZu43pbKGNuj1BjiLsavKHx5v4EYQ5gUzBDLlUMvUUmFNb4lrbulmSw\n"
            "g2Fr13dbjRmgHa5Lj7VAay3xXFwdTGNH3o04uefpvZ/6sRB1e9fP4VR5UVECZLe0\n"
            "D5Au4VSA7usgOLdDjxoG6mBOzEY7vWkbCmbFxB1Q1tWY53ecw9NJ15p8NAtH2dR1\n"
            "+xUeNzDeQMHS4FIZ/Z6c6RuUyusK7fRAxddhUoXu4KVEwbdEV9qsEKDqtW4eUMXX\n"
            "QQBtkxzZkL1lMz4UTXnHwG5jSbHVz3tSyYYpQYZPO2zE/TOrfuZzYGOZ2g9vreNt\n"
            "Ta8u/MMtvhguLV1qCEqAgzDQo0Kx+dc+ueNt/ruCuhWxn0jl0LMX4qvmZX1d2X58\n"
            "J9ub7sFcpLb1gsueYKQ6\n"
            "=RKbN\n"
            "-----END PGP SIGNATURE-----\n",
        )
        self.assertEqual(self.commit.verification.verified, True)
        self.assertEqual(self.commit.verification.verified_at, datetime(2024, 12, 18, 10, 45, 21, tzinfo=timezone.utc))



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Colby Gallup <colbygallup@gmail.com>                          #
# Copyright 2020 Pascal Hofmann <mail@pascalhofmann.de>                        #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class DeploymentStatus(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.deployment = self.g.get_user().get_repo("PyGithub").get_deployment(263877258)
        self.status = self.deployment.get_status(388454671)

    def testAttributes(self):
        self.assertEqual(self.status.created_at, datetime(2020, 8, 26, 14, 32, 51, tzinfo=timezone.utc))
        self.assertEqual(self.status.creator.login, "jacquev6")
        self.assertEqual(
            self.status.deployment_url, "https://api.github.com/repos/jacquev6/PyGithub/deployments/263877258"
        )
        self.assertEqual(self.status.description, "Deployment queued")
        self.assertEqual(self.status.environment, "test")
        self.assertEqual(self.status.environment_url, "https://example.com/environment")
        self.assertEqual(self.status.id, 388454671)
        created_at = datetime(2020, 8, 26, 14, 32, 51, tzinfo=timezone.utc)
        self.assertEqual(self.status.created_at, created_at)
        self.assertEqual(self.status.creator.login, "jacquev6")
        self.assertEqual(
            self.status.deployment_url,
            "https://api.github.com/repos/jacquev6/PyGithub/deployments/263877258",
        )
        self.assertEqual(self.status.description, "Deployment queued")
        self.assertEqual(self.status.environment, "test")
        self.assertEqual(self.status.environment_url, "https://example.com/environment")
        self.assertEqual(self.status.log_url, "https://example.com/deployment.log")
        self.assertEqual(self.status.node_id, "MDE2OkRlcGxveW1lbnRTdGF0dXMzODg0NTQ2NzE=")
        self.assertIsNone(self.status.performed_via_github_app)
        self.assertEqual(
            self.status.repository_url,
            "https://api.github.com/repos/jacquev6/PyGithub",
        )
        self.assertEqual(self.status.state, "queued")
        self.assertEqual(self.status.target_url, "https://example.com/deployment.log")
        self.assertEqual(self.status.updated_at, created_at)
        self.assertEqual(
            self.status.url,
            "https://api.github.com/repos/jacquev6/PyGithub/deployments/263877258/statuses/388454671",
        )
        self.assertEqual(
            self.status.node_id,
            "MDE2OkRlcGxveW1lbnRTdGF0dXMzODg0NTQ2NzE=",
        )
        self.assertEqual(
            repr(self.status),
            'DeploymentStatus(url="https://api.github.com/repos/jacquev6/PyGithub/deployments/263877258/statuses/388454671", id=388454671)',
        )

    def testCreate(self):
        newStatus = self.deployment.create_status(
            "queued",
            target_url="https://example.com/deployment.log",
            description="Deployment queued",
            environment="test",
            environment_url="https://example.com/environment",
            auto_inactive=True,
        )
        self.assertEqual(newStatus.id, 388454671)
        self.assertEqual(newStatus.state, "queued")
        self.assertEqual(
            newStatus.repository_url,
            "https://api.github.com/repos/jacquev6/PyGithub",
        )

    def testGetStatuses(self):
        statuses = self.deployment.get_statuses()
        self.assertListKeyEqual(
            statuses,
            lambda s: s.id,
            [388454671, 388433743, 388432880],
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2026 Matt Davis <35502728+matt-davis27@users.noreply.github.com>   #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import github.PaginatedList
import github.SecretScanAlert

from . import Framework


class SecretScanAlert(Framework.TestCase):
    alert: github.SecretScanAlert.SecretScanAlert

    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("matt-davis27/PyGithub")

    def testAttributes(self):
        alert = self.repo.get_secret_scanning_alert(1)
        self.assertEqual(alert.number, 1)
        self.assertEqual(alert.created_at, datetime(2020, 11, 6, 18, 18, 30, tzinfo=timezone.utc))
        self.assertEqual(alert.url, "https://api.github.com/repos/matt-davis27/PyGithub/secret-scanning/alerts/1")
        self.assertEqual(alert.html_url, "https://github.com/matt-davis27/PyGithub/security/secret-scanning/1")
        self.assertEqual(
            alert.locations_url, "https://api.github.com/repos/matt-davis27/PyGithub/secret-scanning/alerts/1/locations"
        )
        self.assertEqual(alert.state, "open")
        self.assertIsNone(alert.resolution)
        self.assertIsNone(alert.resolved_at)
        self.assertIsNone(alert.resolved_by)
        self.assertEqual(alert.secret_type, "mailchimp_api_key")
        self.assertEqual(alert.secret, "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX-us2")
        self.assertIsNone(alert.push_protection_bypassed_by)
        self.assertFalse(alert.push_protection_bypassed)
        self.assertIsNone(alert.push_protection_bypassed_at)
        self.assertIsNone(alert.push_protection_bypass_request_reviewer)
        self.assertIsNone(alert.push_protection_bypass_request_reviewer_comment)
        self.assertIsNone(alert.push_protection_bypass_request_comment)
        self.assertIsNone(alert.push_protection_bypass_request_html_url)
        self.assertIsNone(alert.resolution_comment)
        self.assertEqual(alert.validity, "unknown")
        self.assertFalse(alert.publicly_leaked)
        self.assertFalse(alert.multi_repo)
        self.assertFalse(alert.is_base64_encoded)
        self.assertEqual(alert.first_location_detected.path, "/example/secrets.txt")
        self.assertEqual(alert.first_location_detected.start_line, 1)
        self.assertEqual(alert.first_location_detected.end_line, 2)
        self.assertEqual(alert.first_location_detected.start_column, 3)
        self.assertEqual(alert.first_location_detected.end_column, 4)
        self.assertFalse(alert.has_more_locations)

    def testMultipleAlerts(self):
        multiple_alerts = self.repo.get_secret_scanning_alerts()
        self.assertIsInstance(multiple_alerts, github.PaginatedList.PaginatedList)
        self.assertIsInstance(multiple_alerts[0], github.SecretScanAlert.SecretScanAlert)
        alert_list = [alert for alert in multiple_alerts]
        self.assertEqual(len(alert_list), 2)

        test_alert = alert_list[
            -1
        ]  # Alerts are returned in descending order, so the first alert is the most recent and the last alert is the oldest
        # Everything below is the same as testAttributes. This is just to make sure the list works.
        self.assertEqual(test_alert.number, 1)
        self.assertEqual(test_alert.created_at, datetime(2020, 11, 6, 18, 18, 30, tzinfo=timezone.utc))
        self.assertEqual(test_alert.url, "https://api.github.com/repos/matt-davis27/PyGithub/secret-scanning/alerts/1")
        self.assertEqual(test_alert.html_url, "https://github.com/matt-davis27/PyGithub/security/secret-scanning/1")
        self.assertEqual(
            test_alert.locations_url,
            "https://api.github.com/repos/matt-davis27/PyGithub/secret-scanning/alerts/1/locations",
        )
        self.assertEqual(test_alert.state, "open")
        self.assertIsNone(test_alert.resolution)
        self.assertIsNone(test_alert.resolved_at)
        self.assertIsNone(test_alert.resolved_by)
        self.assertEqual(test_alert.secret_type, "mailchimp_api_key")
        self.assertEqual(test_alert.secret, "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX-us2")
        self.assertIsNone(test_alert.push_protection_bypassed_by)
        self.assertFalse(test_alert.push_protection_bypassed)
        self.assertIsNone(test_alert.push_protection_bypassed_at)
        self.assertIsNone(test_alert.push_protection_bypass_request_reviewer)
        self.assertIsNone(test_alert.push_protection_bypass_request_reviewer_comment)
        self.assertIsNone(test_alert.push_protection_bypass_request_comment)
        self.assertIsNone(test_alert.push_protection_bypass_request_html_url)
        self.assertIsNone(test_alert.resolution_comment)
        self.assertEqual(test_alert.validity, "unknown")
        self.assertFalse(test_alert.publicly_leaked)
        self.assertFalse(test_alert.multi_repo)
        self.assertFalse(test_alert.is_base64_encoded)
        self.assertEqual(test_alert.first_location_detected.path, "/example/secrets.txt")
        self.assertEqual(test_alert.first_location_detected.start_line, 1)
        self.assertEqual(test_alert.first_location_detected.end_line, 2)
        self.assertEqual(test_alert.first_location_detected.start_column, 3)
        self.assertEqual(test_alert.first_location_detected.end_column, 4)
        self.assertFalse(test_alert.has_more_locations)

    def testRepr(self):
        alert = self.repo.get_secret_scanning_alert(1)
        self.assertEqual(repr(alert), "SecretScanAlert(number=1)")

    def testGetAlertsWithAllArguments(self):
        multiple_alerts = self.repo.get_secret_scanning_alerts(
            state="resolved",
            secret_type="adafruit_io_key",
            resolution="false_positive",
            sort="created",
            direction="asc",
            validity="inactive",
            is_publicly_leaked=False,
            is_multi_repo=False,
            hide_secret=True,
        )
        alert_list = [alert for alert in multiple_alerts]
        self.assertEqual(len(alert_list), 1)

        test_alert = alert_list[-1]
        self.assertIsNone(test_alert.secret)  # Because hide_secret is True



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 AKFish <akfish@gmail.com>                                     #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Justin Kufro <jkufro@andrew.cmu.edu>                          #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Trim21 <trim21.me@gmail.com>                                  #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Bill Napier <napier@pobox.com>                                #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from typing import Any

import github.CodeSecurityConfig
from github.GithubObject import Attribute, NonCompletableGithubObject, NotSet


class DefaultCodeSecurityConfig(NonCompletableGithubObject):
    """
    This class represents a Default Configurations for Code Security.

    The reference can be found here
    https://docs.github.com/en/rest/code-security/configurations.

    The OpenAPI schema can be found at

    - /components/schemas/code-security-default-configurations

    """

    def _initAttributes(self) -> None:
        self._configuration: Attribute[github.CodeSecurityConfig.CodeSecurityConfig] = NotSet
        self._default_for_new_repos: Attribute[str] = NotSet

    def __repr__(self) -> str:
        return self.get__repr__(
            {
                "default_for_new_repos": self.default_for_new_repos,
            }
        )

    @property
    def configuration(self) -> github.CodeSecurityConfig.CodeSecurityConfig:
        return self._configuration.value

    @property
    def default_for_new_repos(self) -> str:
        return self._default_for_new_repos.value

    def _useAttributes(self, attributes: dict[str, Any]) -> None:
        if "configuration" in attributes:  # pragma no branch
            self._configuration = self._makeClassAttribute(
                github.CodeSecurityConfig.CodeSecurityConfig, attributes["configuration"]
            )
        if "default_for_new_repos" in attributes:  # pragma no branch
            self._default_for_new_repos = self._makeStringAttribute(attributes["default_for_new_repos"])



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class GitTree(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.tree = self.g.get_user().get_repo("PyGithub").get_git_tree("f492784d8ca837779650d1fb406a1a3587a764ad")

    def testAttributes(self):
        self.assertEqual(self.tree.sha, "f492784d8ca837779650d1fb406a1a3587a764ad")
        self.assertEqual(len(self.tree.tree), 11)
        self.assertEqual(self.tree.tree[0].mode, "100644")
        self.assertEqual(self.tree.tree[0].path, ".gitignore")
        self.assertEqual(self.tree.tree[0].sha, "8a9af1462c3f4e3358315c2d2e6ef1e7334c59dd")
        self.assertEqual(self.tree.tree[0].size, 53)
        self.assertEqual(self.tree.tree[0].type, "blob")
        self.assertEqual(
            self.tree.tree[0].url,
            "https://api.github.com/repos/jacquev6/PyGithub/git/blobs/8a9af1462c3f4e3358315c2d2e6ef1e7334c59dd",
        )
        self.assertEqual(self.tree.tree[6].mode, "040000")
        self.assertEqual(self.tree.tree[6].path, "ReplayDataForIntegrationTest")
        self.assertEqual(self.tree.tree[6].sha, "60b4602b2c2070246c5df078fb7a5150b45815eb")
        self.assertEqual(self.tree.tree[6].size, None)
        self.assertEqual(self.tree.tree[6].type, "tree")
        self.assertEqual(
            self.tree.tree[6].url,
            "https://api.github.com/repos/jacquev6/PyGithub/git/trees/60b4602b2c2070246c5df078fb7a5150b45815eb",
        )
        self.assertIsNone(self.tree.truncated)
        self.assertEqual(
            self.tree.url,
            "https://api.github.com/repos/jacquev6/PyGithub/git/trees/f492784d8ca837779650d1fb406a1a3587a764ad",
        )

        self.assertEqual(
            repr(self.tree),
            'GitTree(sha="f492784d8ca837779650d1fb406a1a3587a764ad")',
        )
        self.assertEqual(
            repr(self.tree.tree[0]),
            'GitTreeElement(sha="8a9af1462c3f4e3358315c2d2e6ef1e7334c59dd", path=".gitignore")',
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 AetherDeity <aetherdeity+github@gmail.com>                    #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from . import Framework


class Issue140(Framework.TestCase):  # https://github.com/jacquev6/PyGithub/issues/140
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("twitter/bootstrap")

    def testGetDirContentsThenLazyCompletionOfFile(self):
        contents = self.repo.get_contents("js")
        self.assertEqual(len(contents), 15)
        n = 0
        for content in contents:
            if content.path == "js/bootstrap-affix.js":
                self.assertEqual(len(content.content), 4722)  # Lazy completion
                n += 1
            elif content.path == "js/tests":
                self.assertEqual(content.content, None)  # No completion at all
                n += 1
        self.assertEqual(n, 2)

    def testGetFileContents(self):
        contents = self.repo.get_contents("js/bootstrap-affix.js")
        self.assertEqual(contents.encoding, "base64")
        self.assertEqual(
            contents.url,
            "https://api.github.com/repos/twitter/bootstrap/contents/js/bootstrap-affix.js",
        )
        self.assertEqual(len(contents.content), 4722)

    def testGetDirContentsWithRef(self):
        self.assertEqual(
            len(self.repo.get_contents("js", "8c7f9c66a7d12f47f50618ef420868fe836d0c33")),
            15,
        )
